volume_label = 'Rare'
format = 'UDZO'
filesystem = 'HFS+'
size = None
files = ['rare.app']
symlinks = {'Applications': '/Applications'}
icon = 'rare/resources/images/Rare.icns'
background = None
show_status_bar = False
show_tab_view = False
show_pathbar = False
show_sidebar = False
sidebar_width = None
show_icon_preview = True
include_icon_view_settings = 'auto'
include_list_view_settings = 'auto'
arrange_by = None
grid_offset = (0, 0)
grid_spacing = 100
scroll_position = (0, 0)
label_pos = 'bottom'
text_size = 16
icon_size = 128
list_sort_by = None
list_use_relative_dates = None
list_calculate_all_sizes = None
license = {
    'default-language': 'en_US',
    'licenses': {
        'en_US': 'LICENSE'
    },
    'buttons': {
        'en_US': ['English', 'Agree', 'Disagree', 'Print', 'Save', 'License']
    }
}
