# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.7.2
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x02\xa9\
\x00\
\x00\x0a\xb9x\xda\xb5V]O\xa3@\x14}\xe7W\x8c\
5M\xd4X\x05\xba\xb4\xb6\xc6\x97\x8d>\x98\xe8\xaa\xb1\
\xab\x0ff\x1f\x06\xb8\xd0\x89\xe3\x0c\x19.[\xbb\x1b\xff\
\xfb\x0e\x85m\xa1\x0b\xd3\xee\xae6M\xa1\x03s\xce=\
\xe7~\x80u|@&S\x96\x92\x88q \xfaH3\
\x94\xbd\x18\x04(\x8a\x10\x92H\xc9\x17\xd2Iq\xce!\
\x9d\x02\xe0Q2\xef\x1c\x91\xf3\x1b\xf2\xe5fB.\xce\
/';;;\xe4\xe0\xd8\xb2\xeen\xb3t\xfa9C\
\x94\xe2)\xe2\x14\xcf:\xa82\xe8|#?-\xa2?\
\xbeT!\xa81\xb1\x93\xd7\xd3\xcaBO\xd1\x90e\xe9\
\x98x\xcbu\x1a<\xc7Jf\x22\xec\x05\x92K\xbdE\
\xc5>\xdds=\xef\x90\xac~\xbc\xee\xfe\xa9\xf5f\xdd\
]Q\x1f\xf8\xee\xa5\x88\xe4\xe2\xac$+7\xee\x8eF\
\xa3\x024\x92\x02{\x0b\x11c\xc2\x90r\x16T\xd6g\
\xc0\xe2)\x8e\x89\x90\xea\x85\xf2\x05\xecJ\x8b\xc6N\xf5\
\x06^\xfc\xab\x89\xf9\x1d\xdf\xaem\xfb\x9fl\xbb-|\
}yx\x92_6\xe0\x8e\xa7\xf2;\xa86\xf4\x93\xa1\
\x11\xdd\xa3\x9b\xd0C\x96R\x9f\xebl\xb6\x10\xf4\x03#\
\x81\xdb\x10\xfeW\xc16\x1b\x93\xdbb@\xcem\xd9\x88\
l\xb2&7\xc6\x80\x9f\x1b\xb3\x19\xdflNn\x8d\x81\
\xc2]I(J\xf1.\x83\x0c\x1e\xa5z\x06U\xad\xc8\
\x12u\xc6B\x9c\x8e\x89\xb3\xd6\x04ee\xa6\x92\xb3\xb0\
\xb1;\xfa\xf9\x86V\x8e\xa7\xd9\xe2|2O\xe0\xac\xf3\
\x00\x8aE\xf3\xb5\xbe[\xc6;\xf0\xbd\xa1\x1b\xb4\xca\xe9\
{n\xe0\x0c\xb6\xa5\xba\xd6\x99i#r\xed\x817h\
'r\xec\xbe\xdb\xaf\x12\xdd*\x19+H\xd3\xedM[\
N\x8enC/\xfb\x92\x87\xd5\xd6g?\xb4\xbf\xce \
\xc1\xea\xd4\x08\xa4xda\x0cx\x8f\x14\xb3\x1au\x0b\
X\x19\xfel\xca\x10\x8c\xe3\xca>$\xc5w\xb0\x98T\
\x86\xb0\xcbe\x94I\x8fC\x84\xcb\xeb\x8e\xf3\xe7\x0d*\
\x8f\xa7~\x87VSh\xa8\xc8\xb9f\x82\x15g\xf5y\
\xa8#\xdbs\xfb'z\x84\x96?\xfb\xef\xab\xc1\x97\xba\
\xa1^\xea2F\x8d\xb7\xd4\x85\x8c\xba\x8dY\x990\xe4\
\xf0!I\xb1\xebO\x8f\x15\xa3\x94\x1cY\xd2\xf0(\xf9\
\x07\xf0\xea4^24L\xcbEe\x17\xbdO|\xae\
\xc1\x1b\xadv\xec\xee\xdf$\xcb\xcco\x98\xa9\xb1\xa2\xf3\
\x8a7W,\xc5\xed\xb3\xd1\xb0\xed\x03[k\xb8V\x96\
\xdb\x19\xe9\xd5FiE\xdf{\xe4\xfe\xffB\xaadl\
\x15W\xadb\x10^\xb1\xa7__b1&y\x9b5\
*1\xbe\x0d\xd5i\xee\x81C\x80\x0f\x0cf\x86\xf2\x14\
R\xb4\xcaGEE\x9aP\x05\x02\xd7\xb1\x8b\xc3\x15\x13\
p\x11\xb25d3N\xe3\xfbbB\xc3\x90\x89\xb8\x5c\
y\xb3~\x01\xbaga\x94\
"

qt_resource_name = b"\
\x00\x0a\
\x0a\xf6\x0f\xa3\
\x00s\
\x00t\x00a\x00t\x00i\x00c\x00_\x00c\x00s\x00s\
\x00\x0e\
\x03\xc2\xd5\xc3\
\x00s\
\x00t\x00y\x00l\x00e\x00s\x00h\x00e\x00e\x00t\x00.\x00q\x00s\x00s\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x1a\x00\x01\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x92\x09g[\xe6\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
